/*
 * PWM_ATtiny4313.asm
 *
 *  Created: 07.02.2014 15:22:05
 *   Author: Carsten   http://carstenpetercarstensen.homepage.t-online.de/
 * I2C Treiber Quelle: http://www.avrfreaks.net/projects/usi-i2c-slave-attiny-cpus
 * Arithmetik  Quelle: https://www.mikrocontroller.net/articles/AVR_Arithmetik#32_Bit_.2F_32_Bit
 Register
 R0  =alte Tasten		Taste bei der vorigen Abfrage gedrckt
 R1  =ist Jetzt			Taste ist jetzt gedrckt
 R2  =in Arbeit			Taste entweder jetzt oder vorher gedrckt
 R3  =Wartezeit 2Tasten   kurzer oder langer Tastendruck Zhler  von 0..255
 R4  =Wartezeit Taste2
 R5  =Wartezeit Taste3
 R6  =Wartezeit Taste4
 R7  =Wartezeit Taster1
 R8  =Wartezeit Taster2
 R9  =Dimmrichtung 1
 R10 =Dimmrichtung 2
 R11 =Dimmrichtung 3
 R12 =Dimmrichtung 4
 R13 = Sync erforderlich oder nicht
 R14 = 1 kurze Dimmzeit										!0=alle Lampen aus
 R15 = Schleifenzhler
 AL  = R16 Arbeitsregister
 AH  = R17 Arbeitsregister
 */
.include "tn4313def.inc"
.include "avr.inc"

.equ MySlaveAddress	= 0xAE		;I2C Adresse
;.equ Zentralein	= 1				;Eingangsspannung Zentraleingnge in Ruhe: 0= 0V  - 1= VCC 
.equ Quarz		= 1				;0= interner 8Mhz Takt  -  1= externer Quarzoszillator
.equ Flanke		= 1				;Eingnge PD2+3 - 1=steigende  0=fallende   Flanke
.equ PWM8		= 1				;0= PWM Signal Ausgang ATTiny logisch 0 entspricht dunkel	1= PWM Signal Ausgang logisch 1 entspricht hell	- Fr die  8-bit Timer/Counter0
.equ PWM16		= 1				;0= PWM Signal Ausgang ATTiny logisch 0 entspricht dunkel	1= PWM Signal Ausgang logisch 1 entspricht hell	- Fr die 16-bit Timer/Counter1
.equ PWM_Taste  = 0				;0= Taster Licht an entspricht Ausgang =1			1= Taster Licht an entspricht Ausgang =0
.equ WatchdogT	= 1
.equ OffSync	= 192			;Nulldurchgang 50 Hz Wert vom TCNT - bei leerem EEPROM
.equ NS			= 0				;Netzsync bei steigender(0) oder fallender Flanke(1)
.equ OS			= 8				;Offset - je kleiner, je ruhiger
.equ MR			= 20			;Mindestregelung
.equ WS			= 40			;Durchgnge der Warteschleife
.equ kurztaste	= 30			;Schwellwert  kurzer oder langer Tastendruck Dimmer
.equ kurztaste2 = 60			;Schwellwert  kurzer oder langer Tastendruck Favoritentasten
.equ kurztastes = 250			;Schwellwert  Speichern
.equ kurztastew = 30			;verzgerung Dimmen
.equ anaus		= 90			;schaltet unter anaus auf 100% - sonst aus -- kurzer Tastendruck eine Taste

.equ Zeit1 = 1					;Wartezeit ausschalten
.equ Zeit2 = 2
.equ Zeit3 = 3
.equ Zeit4 = 4
.equ Zeit5 = 5
.equ Zeit6 = 6
.equ Zeit7 = 7

.equ Osc   = 0					;0=Oscillator mit 6Mhz fr Netzsynchrones PWM   -   1= Oscillator mit 6,553600Mhz

.if Osc
.equ OscF  = 6553600			;Quarzfrequenz keine Netzsynchronisation mglich
.else
.equ OscF  = 6000000			;Quarzfrequenz mit Netzsynchronisation
.endif

.equ hsec  = 6000				;Fr 50Hz	6000 ;ca. 7320	Fr internen 8Hhz Oszillator 120Hz PWM
.equ fosc  = OscF/100-1			;Quarzfrequenz /100 -1 Letzter Wert vor 0

.equ refr  = 1					;Regelung Frequenz - Schritte pro Durchgang bei groen Abweichungen
.equ refb  = 255				;Begrenzung maximale Regelung, um ein Aufschwingen zu vermeiden

;ab hier bitte I2C nicht ndern
.equ DDR_I2C	= DDRB
.equ PORT_I2C	= PORTB
.equ PIN_I2C	= PINB
.equ SCL		= 7	
.equ SDA 		= 5	

.equ Flag			= GPIOR1
.equ I2Cadr			= 7
.equ I2Crw			= 6			;Read=1
.equ I2CsubAddr		= 5			;WriteSubAddr=1
.equ I2Cask			= 4
;/ab hier bitte I2C nicht ndern
;								PWM entweder invertiert oder nicht invertiert
;8 Bit Zhler PWM
.if PWM8
;.equ	k_pwmmode1 = (1<<COM0A1)|(0<<COM0A0)|(1<<COM0B1)|(0<<COM0B0)|(1<<WGM01)|(1<<WGM00)
.equ	k_pwmmode1 = (0<<COM0A1)|(0<<COM0A0)|(0<<COM0B1)|(0<<COM0B0)|(1<<WGM01)|(1<<WGM00)				;erstmal keine Ausgabe auf den Port
								; Fast PWM-Mode 3 und													entweder -_		Pegel Aus--> 1 An-->0
								;Clear OC0A on Compare Match, set OC0A at TOP
.else
.equ	k_pwmmode1 = (1<<COM0A1)|(1<<COM0A0)|(1<<COM0B1)|(1<<COM0B0)|(1<<WGM01)|(1<<WGM00)
								;Fast PWM-Mode 3 und													oder	 _-		Pegel Aus--> 0 An-->1
;							     Set OC0A on Compare Match, clear OC0A at TOP
.endif
;16 Bit Zhler PWM im Mode 15 also Fast PWM 16 Bit - tccr1a
.if PWM16
;.equ	k_pwmmode2 = (1<<COM1A1)|(0<<COM1A0)|(1<<COM1B1)|(0<<COM1B0)|(1<<WGM11)|(0<<WGM10)	
.equ	k_pwmmode2 = (0<<COM1A1)|(0<<COM1A0)|(0<<COM1B1)|(0<<COM1B0)|(1<<WGM11)|(0<<WGM10)	
								;COM1A1 COM1A0 COM1B1 COM1B0   WGM11 WGM10 --> Register TCCR1A       entweder -_		Pegel Aus--> 1 An-->0
.else
.equ	k_pwmmode2 = (1<<COM1A1)|(1<<COM1A0)|(1<<COM1B1)|(1<<COM1B0)|(1<<WGM11)|(0<<WGM10)
								;COM1A1 COM1A0 COM1B1 COM1B0   WGM11 WGM10 --> Register TCCR1A     	oder	 _-		Pegel Aus--> 0 An-->1
.endif

.equ	k_vorteiler1 = 0b00000100	;TCCR0B, Teilerfaktor Timer0 = 0b00000001 1:1  0b00000010 1:8	0b00000011 1:64   0b00000100 1:256  0b00000101 1:1024
.equ	k_vorteiler2 = (0<<ICNC1)|(0<<ICES1)|(1<<WGM13)|(1<<WGM12)|(0<<CS12)|(0<<CS11)|(1<<CS10)	;Fast PWM
									;0b00111001	;TCCR1B - ICNC1 ICES1  WGM13 WGM12 CS12 CS11 CS10	
									;TCCR1B, Teilerfaktor Timer0 = 0bxxxxx001 1:1  0bxxxxx010 1:8	0bxxxxx011 1:64   0bxxxxx100 1:256  0bxxxxx101 1:1024

.equ	k_TIMSK = 0b10000000		;Interrupt bei berlauf - 8 Bit Timer 0 fr Uhr = Bit 1 - 16 Bit Timer 1 fr 50 Hz Sync 0b10000010




.ESEG				; EEPROM-Bereich
daten_eeprom:

favorit1:			.byte 6			;Tastenkombination fr Taste 1+2
favorit2:			.byte 6			;Tastenkombination fr Taste 1+3
favorit3:			.byte 6			;Tastenkombination fr Taste 1+4
favorit4:			.byte 6			;Tastenkombination fr Taste 1+5
favorit5:			.byte 6			;Tastenkombination fr Taste 1+6
favorit6:			.byte 6			;Tastenkombination fr Taste 2+3
favorit7:			.byte 6			;Tastenkombination fr Taste 2+4
favorit8:			.byte 6			;Tastenkombination fr Taste 2+5
favorit9:			.byte 6			;Tastenkombination fr Taste 2+6
favorit10:			.byte 6			;Tastenkombination fr Taste 3+4
favorit11:			.byte 6			;Tastenkombination fr Taste 3+5
favorit12:			.byte 6			;Tastenkombination fr Taste 3+6
favorit13:			.byte 6			;Tastenkombination fr Taste 4+5
favorit14:			.byte 6			;Tastenkombination fr Taste 4+6
favorit15:			.byte 6			;Tastenkombination fr Taste 5+6
favorit16:			.byte 6			;bel. Doppeltasten langer Tastendruck
minimal:			.byte 6			;Minimalwerte der Dimmer 4x 
etabd:				.byte 1			;Dimmkurve Dimmer 1 bis 4
EWF0:				.byte 144		;Bedingt+Wochentag/Minuten/Stunden/Monat/Tag/Faforit		-Schaltzeit
ECWert:				.byte 1			;fr 50Hz sync

;=========================================================;
.dseg
.org	SRAM_START					;RAMTOP

I2CsubAddrBuf:	 	.byte 1			
status:				.byte 1
favorit:			.byte 1
soll1:				.byte 1
soll2:				.byte 1
soll3:				.byte 1
soll4:				.byte 1
soll5:				.byte 1
soll6:				.byte 1
dmin1:				.byte 1			;Minimaler Dimmwert Dimmer 1 -  8 Bit
dmin2:				.byte 1			;Minimaler Dimmwert Dimmer 2 -  8 Bit
dmin3:				.byte 1			;Minimaler Dimmwert Dimmer 3 - 16 Bit
dmin4:				.byte 1			;Minimaler Dimmwert Dimmer 4 - 16 Bit
ist1:				.byte 1			;Ist 0..200 - Dimmer 1
ist2:				.byte 1			;Ist 0..200 - Dimmer 2
ist3:				.byte 1			;Ist 0..200 - Dimmer 3
ist4:				.byte 1			;Ist 0..200 - Dimmer 4
ist5:				.byte 1			;0 oder 1 - an oder aus
ist6:				.byte 1			;0 oder 1 - an oder aus
dk:					.byte 1			;Dimmkurve Dimmer 1-Bit0 bis Dimmer 4-Bit3  - 0=linear 1=log
Minute:				.byte 1			;Uhrzeit
Stunde:				.byte 1	
Tag:				.byte 1	
Monat:				.byte 1
Wochentag:			.byte 1
zeith:				.byte 1			;bis 1 sec		  H
zeitl:				.byte 1			;Zhlt 1/100 sec  L
nminute:			.byte 1			;Flag fr neue Minute
Zeit:				.byte 1			;nderung - Neue Zeit
WF0:				.byte 192		;Aktiv+Wochentag/Minuten/Stunden/Monat/Tag/Faforit		-Schaltzeit 6 Byte * 32 Zeiten

CWert:				.byte 1			;16-bit Timer/Counter soll TCNT1 fr 50Hz sync - Sollwert
altr:				.byte 1			;letzte Richtung 50 Hz korrektur ber ICR1
altd:				.byte 1			;Abweichung 50 Hz zwischen Richtungsnderung

Soll:				.byte 1			;Zwischenwert
Ist:				.byte 1			;Zwischenwert
dmin:				.byte 1			;Zwischenwert
Kurve:				.byte 1			;Zwischenwert
alt_soll1:			.byte 1			;letzter soll
alt_soll2:			.byte 1
alt_soll3:			.byte 1
alt_soll4:			.byte 1
alt_soll5:			.byte 1
alt_soll6:			.byte 1

Tabram:
;****************************************************************************************

.cseg
.ORG 0
	RJMP RESET
.ORG INT0addr
	reti
.ORG INT1addr
	reti
.ORG OVF1addr			;16 Bit Timer berlauf
	rjmp Synchron
;	reti
.ORG OVF0addr			;8  Bit Timer berlauf
	reti
.ORG PCIBaddr			;= 0x000b	; Pin Change Interrupt Request B
	rjmp NSync
.ORG OC0Aaddr
	reti
.ORG USI_STARTaddr		;USI_STRaddr
	rjmp StartCondition
.ORG USI_OVFaddr
	rjmp I2CdataRX
.ORG WDTaddr			;Watchdog Timer Overflow
	RJMP WDR_RESET
;-------------
;I2C
StartCondition:
	push AL
	in AL,SREG
	push AL

 	cbi DDR_I2C,SDA
	outi Flag,0

	sbic PIN_I2C,SCL	;Wait for SCL to go low to ensure the "Start Condition" has completed.
	rjmp PC-1

	outi USICR,(1<<USISIE)|(1<<USIOIE)|(1<<USIWM1)|(1<<USIWM0)|(1<<USICS1)   
   	outi USISR,(1<<USISIF)|(1<<USIOIF)|(1<<USIPF)
  
	pop	AL
	out	SREG,AL
	pop AL
	reti
;----------------------------------------------------------;
I2CdataRX:
	pushw Y
	pushw A
	in AL,SREG
	push AL

	in AH,USIDR

	sbis Flag,I2Cadr
	rjmp CheckAddress

	sbis Flag,I2Crw
	rjmp WriteData
;------------------------------------
;ReadData:
	sbis Flag,I2Cask
	rjmp ReadData1

	sbis Flag,I2CsubAddr
	rjmp Ask0
	cpi AH,0
	brne ResetUSI			;NACK
PutData:
	cbi Flag,I2Cask
	lds YL,I2CsubAddrBuf
	clr YH
	addiw Y,SRAM_START
	ld AL,Y+
	out USIDR,AL
	subiw Y,SRAM_START
	sts I2CsubAddrBuf,YL

	sbi DDR_I2C,SDA			;Set SDA as output
   	ldi AL,(1<<USISIF)|(1<<USIOIF)|(1<<USIPF)
	rjmp I2CdataRX_outAL
Ask0:
	sbi Flag,I2CsubAddr
	rjmp PutData
ReadData1:
	sbi Flag,I2Cask
	cbi DDR_I2C,SDA			;Set SDA as input
	outi USIDR,0			;Prepare ACK 
	rjmp I2CdataRX_out7E
;------------------------------------
WriteData:
	sbis Flag,I2Cask
	rjmp WriteData0
	cbi Flag,I2Cask 
 	cbi DDR_I2C,SDA
   	ldi AL,(1<<USISIF)|(1<<USIOIF)|(1<<USIPF)
	rjmp I2CdataRX_outAL
ResetUSI:
 	cbi DDR_I2C,SDA
	outi Flag,0
	outi USICR,(1<<USISIE)|(1<<USIWM1)|(0<<USIWM0)|(1<<USICS1); ;Start Condition Interrupt;Set USI in Two-wire mode;Clock Source = External, positive edge        */  \
    ldi AL,(1<<USISIF)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)		;Clear all flags, except Start Cond 
	rjmp I2CdataRX_outAL
WriteData0:
	sbic Flag,I2CsubAddr
	rjmp DoData
;Save SubAddress
	sts I2CsubAddrBuf,AH
	sbi Flag,I2CsubAddr
	sbi Flag,I2Cask
	rjmp Send_ASK
DoData:
	lds YL,I2CsubAddrBuf
	clr YH
	addiw Y,SRAM_START
	st Y+,AH
	subiw Y,SRAM_START
	sts I2CsubAddrBuf,YL
	sbi Flag,I2Cask
	rjmp Send_ASK
;---------------------------------
CheckAddress:
	bst AH,0
	cbr AH,1
	cpi AH,MySlaveAddress
	brne ResetUSI
	ldi AL,(1<<I2Cadr)|(1<<I2Cask)
	bld AL,I2Crw
	out Flag,AL
Send_ASK:
	outi USIDR,0
	sbi DDR_I2C,SDA
I2CdataRX_out7E:
  	ldi AL,(1<<USISIF)|(1<<USIOIF)|(1<<USIPF)|(0x0E<<USICNT0)
I2CdataRX_outAL:
   	out USISR,AL

	pop	AL
	out	SREG,AL
	popw A
	popw Y
	reti
;endeI2C

;----------------------------------------------------------
NSync:						;Zur Syncronisierung 50Hz - PCI - in R13 die Abweichung TCNT1 zum CWert - Bit 7 zeigt die Korrekturrichtung
.if NS
	sbic PINB, 6			;nur beim Impuls =0 prfen PB6
.else
	sbis PINB, 6			;nur beim Impuls =1 prfen
.endif
	reti

	push AL
	in AL,SREG
	push AL
	push AH
	push R26
	push R27
	clr AL
	in R27, TCNT1L
	in R26, TCNT1H
	lds AH, CWert
	sub AL, R27
	sbc AH, R26
	in R26, ICR1L
	in R27, ICR1H
	breq NSyncx				;Werte sind gleich
	brcs NSync1											;?
	ldi AL, 1
	rjmp NSync3
NSync1:
	ldi AL, 2
NSync3:
	lds AH, altr
	cp AL, AH
	breq NSync5
	sts altr, AL
	cpi AL, 2
	brcc NSync9
	lds AL, altd
	lsr AL					;/2
	clr AH
	sts altd, AH			;lschen
	rjmp NSync4
NSync9:
	lds AL, altd
	lsr AL					;/2
	clr AH
	sts altd, AH			;lschen
	rjmp NSync8
NSync5:
	cpi AL, 2
	ldi AH,HIGH (refr)
	ldi AL,LOW (refr)
	brcs NSync4
NSync8:
	add R26, AL
	adc R27, AH
	ldi AH,HIGH (fosc+refb)	;Maximalen Drift begrenzen
	ldi AL,LOW (fosc+refb)
	sub AL, R26
	sbc AH, R27
	brcs NSyncx				;Drift begrenzen
	rjmp NSync2
NSync4:
	sub R26, AL
	sbc R27, AH
	ldi AH,HIGH (fosc-refb)	;Maximalen Drift begrenzen
	ldi AL,LOW (fosc-refb)
	sub AL, R26
	sbc AH, R27	
	brcc NSyncx	
NSync2:
	ori R27, 0xE0			;nicht kleiner
	OUT ICR1H, R27
	OUT ICR1L, R26
NSyncx:
	lds AH, altd		;+1
	ldi AL, refr
	add AH, AL
	brcc NSyncue
	ldi AH, 255			;bei berlauf 255
NSyncue:
	sts altd, AH
	clr R13
NSyncux:
	pop	R27
	pop	R26
	pop	AH
	pop	AL
	out	SREG,AL
	pop	AL
	reti
;--------------------------
Synchron:
	push AL
	in AL,SREG
	push AL
	push AH
	push R24
	push R25

	inc R13					;Standardwerte einstellen, wenn 2,5 sec keine Sync Impulse kommen
	brne Synchronx
	ldi AH,HIGH (fosc)		;
	ldi AL,LOW (fosc)
	OUT ICR1H, AH
	OUT ICR1L, AL
 Synchronx:
							;Uhrensteuerung
 	ldi AL, LOW (hsec)
	ldi AH, HIGH(hsec)
	lds R24, zeitl
	lds R25, zeith
	adiw R24, 1
	sub AL, R24
	sbc AH, R25
	brcc Zeit01
	clr R24
	clr R25
	lds AL, nminute
	ori AL, 0b01111111		;Bit 7 zufrieden lassen
	sts nminute, AL
Zeit01:
	sts zeitl, R24
	sts zeith, R25

	pop R25
	pop R24
	pop	AH
	pop	AL
	out	SREG,AL
	pop	AL
	reti
;----------------------------------------------------------;
;ende ISR
;----------------------------------------------------------;
warte:
	ldi AL, WS				;schleifendurchgnge
warte1:
	rcall schleife			; innere Schleife
	dec AL
	brne warte1
	ret
schleife:
	ldi AH, 0xFF			;innere Wasteschleife
schleife1:
	dec AH
	brne schleife1
	ret
;--------------------------------------------------------
;Tasten
taste1:	
	ldi AL, 0b00000100
	and AL, R1				;ist jetzt die Taste gddrckt
	breq taste1d
	ldi AL, 0xFF			;Taste ist gedrckt
	cp AL, R3				;max erreicht? R3= Wartezeit Taste1
	breq taste1c			;ja - weiter
	inc R3
	rjmp taste1c
taste1d:					;Taste nicht gedrckt. Vorher?
	ldi AL, kurztaste		;Kurzer Tastendruck?
	sub AL, R3
	brcs taste1e			;Sprung - langer Tastendruck
							;kurzer Tastendruck - hell schalten	bei R9=1 sonst aus bei R9=FF
	ldi AL, 0x00			;aus
	lds AH, soll1
;	cpi AH, 50				;< oder > 50%
	cpi AH, anaus			;< oder > anaus
	brcc taste1d1
	ldi AL,100
taste1d1:
	sts soll1, AL
	sts alt_soll1, AL
	ldi AL, 0b00000001		;Flagregister R14 setzen fr schnelle Helligkeitsnderung
	or AL, R14
	mov R14, AL
taste1e:					;langer Tastendruck - Taste nicht gedrckt
	clr R3					;Zhler kurzer Tastendruck = 0
	mov R0, R1
	ret
 taste1c:					;Dimmen  -  Soll1 +-1
	mov AH, R9				;AL = Soll - AH = Richtung
	lds AL, soll1			;SRAM-Adresse in AL
	rcall Tastew			; +- 1 - Richtung
	mov R9, AH				;sichern
	sts soll1, AL
	sts alt_soll1, AL
	mov R0, R1
	ret
;-------------
taste2:
	ldi AL, 0b00001000
	and AL, R1				;ist jetzt die Taste gddrckt
	breq taste2d
	ldi AL, 0xFF			;Taste ist gedrckt
	cp AL, R3				;max erreicht? R4= Wartezeit Taste2
	breq taste2c			;ja - weiter
	inc R3
	rjmp taste2c
taste2d:					;Taste nicht gedrckt. Vorher?
	ldi AL, kurztaste		;Kurzer Tastendruck?
	sub AL, R3
	brcs taste2e			;Sprung - langer Tastendruck
							;kurzer Tastendruck - hell schalten	bei R9=1 sonst aus bei R9=FF
	ldi AL, 0x00			;aus
	lds AH, soll2
;	cpi AH, 50				;< oder > 50%
	cpi AH, anaus			;< oder > anaus
	brcc taste2f
	ldi AL,100
taste2f:
	sts soll2, AL
	sts alt_soll2, AL
	ldi AL, 0b00000010		;Flagregister R14 setzen
	or AL, R14
	mov R14, AL
taste2e:					;langer Tastendruck - Taste nicht gedrckt
	clr R3					;Zhler kurzer Tastendruck = 0
	mov R0, R1
	ret
 taste2c:					;Dimmen  -  Soll1 +-1
	mov AH, R10				;AL = Soll - AH = Richtung
	lds AL, soll2			;SRAM-Adresse in AL
	rcall Tastew			; +- 1 - Richtung
	mov R10, AH				;sichern
	sts soll2, AL
	sts alt_soll2, AL
	mov R0, R1
	ret
;-------------
taste3:
	ldi AL, 0b00010000
	and AL, R1				;ist jetzt die Taste gddrckt
	breq taste3d
	ldi AL, 0xFF			;Taste ist gedrckt
	cp AL, R3				;max erreicht? R4= Wartezeit Taste2
	breq taste3c			;ja - weiter
	inc R3
	rjmp taste3c
taste3d:					;Taste nicht gedrckt. Vorher?
	ldi AL, kurztaste		;Kurzer Tastendruck?
	sub AL, R3
	brcs taste3e			;Sprung - langer Tastendruck
							;kurzer Tastendruck - hell schalten	bei R9=1 sonst aus bei R9=FF
	ldi AL, 0x00			;aus
	lds AH, soll3
;	cpi AH, 50				;< oder > 50%
	cpi AH, anaus			;< oder > anaus
	brcc taste3f
	ldi AL,100
taste3f:
	sts soll3, AL
	sts alt_soll3, AL
	ldi AL, 0b00000100		;Flagregister R14 setzen
	or AL, R14
	mov R14, AL
taste3e:					;langer Tastendruck - Taste nicht gedrckt				
	clr R3					;Zhler kurzer Tastendruck = 0
	mov R0, R1
	ret
 taste3c:					;Dimmen  -  Soll1 +-1
	mov AH, R11				;AL = Soll - AH = Richtung
	lds AL, soll3			;SRAM-Adresse in AL
	rcall Tastew			; +- 1 - Richtung
	mov R11, AH				;sichern
	sts soll3, AL
	sts alt_soll3, AL
	mov R0, R1
	ret
;-------------
taste4:
	ldi AL, 0b01000000
	and AL, R1				;ist jetzt die Taste gddrckt
	breq taste4d
	ldi AL, 0xFF			;Taste ist gedrckt
	cp AL, R3				;max erreicht?
	breq taste4c			;ja - weiter
	inc R3
	rjmp taste4c
taste4d:					;Taste nicht gedrckt. Vorher?
	ldi AL, kurztaste		;Kurzer Tastendruck?
	sub AL, R3
	brcs taste4e			;Sprung - langer Tastendruck
							;kurzer Tastendruck - hell schalten	bei R9=1 sonst aus bei R9=FF
	ldi AL, 0x00			;aus
	lds AH, soll4
;	cpi AH, 50				;< oder > 50%
	cpi AH, anaus			;< oder > anaus
	brcc taste4f
	ldi AL,100
taste4f:
	sts soll4, AL
	sts alt_soll4, AL
	ldi AL, 0b00001000		;Flagregister R14 setzen
	or AL, R14
	mov R14, AL
taste4e:					;langer Tastendruck - Taste nicht gedrckt
	clr R3					;Zhler kurzer Tastendruck = 0
	mov R0, R1
	ret
taste4c:					;Dimmen  -  Soll1 +-1
	mov AH, R12				;AL = Soll - AH = Richtung
	lds AL, soll4			;SRAM-Adresse in AL
	rcall Tastew			; +- 1 - Richtung
	mov R12, AH				;sichern
	sts soll4, AL
	sts alt_soll4, AL
	mov R0, R1
	ret
;----
taste5:						;Funktion Stromstoschalter
	tst R1					;ist die Taste gedrckt?
	breq taste5b
	tst R0					;wurde vorher die Taste gedrckt?
	brne taste5a
	lds AL, soll5
	com AL					;umschalten
	andi AL, 0b00000001
	sts soll5, AL
	mov R0, R1				;nur einmal umschalten
	ret
taste5a:
	mov R0, R1				;nur einmal umschalten
	ldi AL, 0xFF			;Taste ist gedrckt
	cp AL, R3				;max erreicht?
	breq taste6d			;ja - weiter
	ldi AL, 0xF0
	cp AL, R3				;Schreibwert erreicht?
	breq taste5c
	ret
taste5b:					;Taste nicht gedrckt
	ldi AL, 0				;fr Status = 0
	rjmp kt1e				;alles lschen und zurck
taste5c:
		rcall ein
taste6:
	tst R1					;ist die Taste gedrckt?
	breq taste6b
	tst R0					;wurde vorher die Taste gedrckt?
	brne taste6a
	lds AL, soll6
	com AL					;umschalten
	andi AL, 0b00000001
	sts soll6, AL
	rcall taste6d
taste6a:
	ldi AL, 0xFF			;Taste ist gedrckt
	cp AL, R3				;max erreicht?
	breq taste6d			;ja - weiter
	ldi AL, 0xF0
	cp AL, R3				;Schreibwert erreicht?
	breq taste6c
taste6d:
	mov R0, R1				;nur einmal umschalten
	ret
taste6c:					;Maxwerte im EEPROM
	rjmp setmax				;ins EEPROM schreiben
taste6b:					;Taste nicht gedrckt
	ldi AL, 0				;fr Status = 0
	rjmp kt1e				;alles lschen und zurck
Tastew:						;AL = Soll - AH = Richtung
	ldi R18, kurztastew		;wartezeit bevor gedimmt wird
	cp R18, R3
	brcc Tastew2
	inc R15					;Schleifenzhler
	sbrc R15, 0				;mur jeder 2. Durchgang
	ret
Tastewg:
	cpi AL, 0				;Teste, ob 0
	breq Tastew3
	add AL, AH				; Soll +- 1
	cpi AL, 101				;>100?
	brcs Tastew1
	ldi AL, 100				;max 100
	ldi AH, 0xFF			;Dimmrichtung rckwrts
	ret
Tastew3:					;soll 0 + 1
	inc AL
	ldi AH, 1				;Dimmrichtung vorwerts
	ret
Tastew1:
	cpi AL, 0
	brne Tastew2
	ldi AH, 1
Tastew2:
	ret
;----------------------------------
Tdb:
	mov R0, R1				;alten Wert sichern
	clr R2					;Keine Taste gedrckt - R2 lschen
	ret
; Tastenabfrage
Td:
	in AL, PIND
	andi AL, 0b01011111		;Taster PD0,PD1,PD2,PD3,PD4,PD6 filtern
	mov R1, AL				;Ist in R1
	breq Ta					;Sprung(Z=0), Ist = keine Taste gedrckt, Abfrage nderung
	eor AL, R0				;Tasten gendert? - EXOR mit letztem Zustand
	rjmp Tw0
Ta:	eor AL, R0				;Tasten gendert? - EXOR mit letztem Zustand
	breq Tdb				;Keine Taste gedrckt und keine nderung --> MainLoop	Sprung nach Twx zu weit entfernt!
Tw0:	
	mov AL, R1				;ist
	or AL, R0				;ist oder alter Wert
Twa0:
	or R2, AL				;nderung in R2			
	cpi AL, 0b00000100		;Taste PD2? - UND
	brne TW1				;springe, wenn nicht Taste1 gesetzt
Tw0a:
	ldi AL, 0
	sts status, AL	
	rjmp taste1				;nur eine Taste wurde gedrckt
TW1:
	mov AL, R2
	cpi AL, 0b00001000		;Taste 2 - PD3?
	brne TW2
Tw1a:
	ldi AL, 0
	sts status, AL		
	rjmp taste2
TW2:
	cpi AL, 0b00010000		;Taste PD4?
	brne TW3
Tw2a:
	ldi AL, 0
	sts status, AL	
	rjmp taste3
TW3:	
	cpi AL, 0b01000000		;Taste PD6?
	brne TW4
Tw3a:
	ldi AL, 0
	sts status, AL	
	rjmp taste4
TW4:						;Taste auf PD0
	cpi AL, 0b00000001		;Taste 1?
	brne TW5
Tw4a:
	ldi AL, 0
	sts status, AL	
	rjmp taste5
TW5:						;Taste auf PD1
	cpi AL, 0b00000010		;Taste 2?
	brne TWY
Tw5a:
	ldi AL, 0
	sts status, AL	
	rjmp taste6
TWY:						;Tastenkombinationen
	ldi AL, 0b00001100		;Taste 1+2
	cp AL, R2
	brne TWY2				;Sprung wenn nicht gleich
	ldi AH, favorit1
	ldi AL, 1
	sts favorit, AL
	rjmp kt
TWY2:
	ldi AL, 0b00010100		;Taste 1+3
	cp AL, R2
	brne TWY3				;Sprung wenn nicht gleich
	ldi AH, favorit2
	ldi AL, 2
	sts favorit, AL
	rjmp kt
TWY3:
	ldi AL, 0b01000100		;Taste 1+4
	cp AL, R2
	brne TWY4				;Sprung wenn nicht gleich
	ldi AH, favorit3
	ldi AL, 3
	sts favorit, AL
	rjmp kt
TWY4:
	ldi AL, 0b00000101		;Taste 1+5
	cp AL, R2
	brne TWY5				;Sprung wenn nicht gleich
	ldi AH, favorit4
	ldi AL, 4
	sts favorit, AL
	rjmp kt
TWY5:
	ldi AL, 0b00000110		;Taste 1+6
	cp AL, R2
	brne TWY6				;Sprung wenn nicht gleich
	ldi AH, favorit5
	ldi AL, 5
	sts favorit, AL
	rjmp kt
TWY6:
	ldi AL, 0b00011000		;Taste 2+3
	cp AL, R2
	brne TWY7				;Sprung wenn nicht gleich
	ldi AH, favorit6
	ldi AL, 6
	sts favorit, AL
	rjmp kt
TWY7:
	ldi AL, 0b01001000		;Taste 2+4
	cp AL, R2
	brne TWY8				;Sprung wenn nicht gleich
	ldi AH, favorit7
	ldi AL, 7
	sts favorit, AL
	rjmp kt
TWY8:
	ldi AL, 0b00001001		;Taste 2+5
	cp AL, R2
	brne TWY9				;Sprung wenn nicht gleich
	ldi AH, favorit8
	ldi AL, 8
	sts favorit, AL
	rjmp kt
TWY9:
	ldi AL, 0b00001010		;Taste 2+6
	cp AL, R2
	brne TWY10				;Sprung wenn nicht gleich
	ldi AH, favorit9
	ldi AL, 9
	sts favorit, AL
	rjmp kt
TWY10:
	ldi AL, 0b01010000		;Taste 3+4
	cp AL, R2
	brne TWY11				;Sprung wenn nicht gleich
	ldi AH, favorit10
	ldi AL, 10
	sts favorit, AL
	rjmp kt
TWY11:
	ldi AL, 0b00010001		;Taste 3+5
	cp AL, R2
	brne TWY12				;Sprung wenn nicht gleich
	ldi AH, favorit11
	ldi AL, 11
	sts favorit, AL
	rjmp kt
TWY12:
	ldi AL, 0b00010010		;Taste 3+6
	cp AL, R2
	brne TWY13				;Sprung wenn nicht gleich
	ldi AH, favorit12
	ldi AL, 12
	sts favorit, AL
	rjmp kt
TWY13:
	ldi AL, 0b01000001		;Taste 4+5
	cp AL, R2
	brne TWY14				;Sprung wenn nicht gleich
	ldi AH, favorit13
	ldi AL, 13
	sts favorit, AL
	rjmp kt
TWY14:
	ldi AL, 0b01000001		;Taste 4+6
	cp AL, R2
	brne TWY15				;Sprung wenn nicht gleich
	ldi AH, favorit14
	ldi AL, 14
	sts favorit, AL
	rjmp kt
TWY15:
	ldi AL, 0b00000011		;Taste 5+6
	cp AL, R2
	brne TWY16				;Sprung wenn nicht gleich
	ldi AH, favorit15
	ldi AL, 15
	sts favorit, AL
	rjmp kt
;bis hier sind alle mglichen Eizeltasten und Zweierkombinationen abgefragt

TWY16:						;3 Tasten gedrckt?
	clr AL
	sbic PIND, 0
	inc AL
	sbic PIND, 1
	inc AL
	sbic PIND, 2
	inc AL
	sbic PIND, 3
	inc AL
	sbic PIND, 4
	inc AL
	sbic PIND, 6
	inc AL
	cpi AL, 3
	brne TWX				;nicht 3 Tasten gedrckt
	ldi AH, favorit16
	ldi AL, 16
	sts favorit, AL
	rjmp kt
TWX:						;fertig
	mov R0, R1				;alten Wert sichern
Twz:
	ret
;ende Tastenabfrage
;-------------
kt:							;Kominationstasten (mindestens 2 gleichzeitig)
	ldi AL, 1
	sts status, AL			;Status auf 1 setzen
	clr R13
	dec R13					;int setzen

	ldi AL, 0
	or AL, R1				;ist jetzt eine weitere Taste gddrckt
	breq kt1c				;Sprung, wenn keine Taste gedrckt

	ldi AL, kurztastes		;
	cp AL, R3
	breq kt1f

	ldi AL, 0xFF			;
	cp AL, R3				;max erreicht? R3= Wartezeit Tasten
	breq kt1b				;ja - weiter zum schreiben
	inc R3	
kt1b:						;zurck														
	mov R0, R1
	ret
kt1c:						;keine Taste gedtckt
	ldiw X, soll1
	ldi AL, kurztaste2		;vorher kurzer oder langer Tastendtuck?
	cp AL, R3
	brcs kt1d
	;AH = Adresse im EEPROM - Doppelregister X = Ziel ;6 Werte werden gelesen
	rcall neu_nach_alt
	lds AL, favorit
	rcall est				;fav-Nr in AL - Adresse in AH zurck
	rcall lesen
	rcall alt_nach_neu
	ldi AL, 1				;Status
	rjmp kt1e
kt1d:
	lds AL, soll1			;sind alle Lampen aus?
	lds AH, soll2
	or AL, AH
	lds AH, soll3
	or AL,AH
	lds AH, soll4
	or AL, AH
	lds AH, soll5
	andi AH, 1
	or AL, AH
	lds AH, soll6
	andi AH, 1
	or AL, AH
	brne kt1a
	ldi AH, favorit16							
	rcall lesen				;AH = Adresse im EEPROM - Doppelregister X = Ziel
	ldi AL, 16
	sts favorit, AL
	ldi AL, 3
	rjmp kt1e
kt1a:
	rcall aus
	ldi AL, 2				;Status
kt1e:						;Rcksprung
 	sts status, AL			;Status setzen
	clr R0					;Clear Register
	clr R1
	clr R2
	clr R3					;R3  =Wartezeit Taste
	ret
kt1f:
	sts favorit, R0			;Favoritentastenkombination neu setzen
	;Doppelregister X = Quelle ---> AH = Ziel im EEPROM
	ldiw X, soll1
	rcall schreiben			;Werte ins EEPROM schreiben
	ldi AL, 0
	sts ist1, AL
	sts ist2, AL
	sts ist3, AL
	sts ist4, AL
	clr R0					;Clear Register
	clr R1
	clr R2
kt1g:
	inc R3
kt1w:
	mov R0, R1
	ret
;------------------------------------------------
alt_nach_neu:
	clr R20
	ldiw y, soll1
	ldiw z, alt_soll1
ersetze:
	ld R19, y				;soll
	cpi R19, 255
	brne ersetze1
	ld R19, z
	st y, R19
ersetze1:
	adiw y, 1				;+1
	adiw z, 1				;+1
	inc R20
	cpi R20, 6
	brne ersetze
	ret
;------------------------------------------------
neu_nach_alt:
	clr R20
	ldiw y, soll1
	ldiw z, alt_soll1
verschiebe:
	ld R19, y+
	cpi R19, 255
	breq verschiebe1
	st z, R19
verschiebe1:
	adiw z, 1				;+1
	inc R20
	cpi R20, 6
	brne verschiebe
	ret
;------------------------------------------------
wert:	;AL = soll (0..100%) ; AH = ist (0..200 Offset) ; R20 - Anfangswert (0..100%) ; R18 - Dimmkurve
;Wertebereich begrenzen
	cpi AL, 100		;soll
	brcs wertw
	ldi AL, 100
;AL = Soll ; AH = Ist ; R19 = dmin - Anfangswert ; R18 - Kurve
					;Delta = (100-dmin)*2			<Schritte>
					;(Delta*Soll/100)+(dmin*2)	<Zeiger Tabelle>
wertw:
	lds R19, dmin
	ldi R20, 100			;Delta =100
	sub R20, R19			; - dmin

	clr R21
	clc						;Clear Carry
	rol R20					;R20,R21 *2
	rol R21
	clc						;Clear Carry
	rol R20					;R20,R21 *2
	rol R21
	clr AH
	rcall WertP				;AL,AL * R20 /200	-	Delta*Soll/200		,R23..R20 (Rest)
	tst AL					;ist AL >0
	brne Wert1
	lds AL, soll
Wert1:
	lds R20, dmin			; + Anfangswert
	clr R21
	clc						;Clear Carry
	rol R20					;R20,R21 *2
	rol R21
	add R20, R16			; +
	adc R21, R17			;in R20 steht der Soll-Wert (0...200 = C8 Hex)
	lds AH, ist				;(0...200)
	lds R21, soll			;Prfe, ob soll und ist =0
	or R21, AH
	brne Wert2
	ret						;beides 0 - mit Z zurck
Wert2:
	tst AL					;aus?
	brne Wert3				;Springe, wenn nicht 0
	clr R20					;sonst setze auf 0
Wert3:
	cp R20, AH
	breq wt2				;keine nderung
	ldi AL, 0x01			; add 255 = dec ; add 1 = inc
	brcc wt0				;C=0 dann inc AL 
	neg AL					; add neg 255 (=1) = inc
	add AL, AH				;+-2
	lds R20, dmin			;Anfangswert
	lsl R20					;*2
	cp AL, R20				;Teste, ob unter dmin
	brcc wertb
	clr AL
	sts ist, AL				;neuer ist Wert
;	ser AL					;0xFF ist licht aus
;	ser AH
	clr AL					;0x00 ist licht aus
	clr AH
	clz						;Clear Zero Flag
	ret
wt0:
	add AL, AH				;+-1 (=0...200)
	lds R20, dmin			;Anfangswert (0...100)
	lsl R20					; *2
	cp AL, R20				;Teste, ob unter dmin
	brcc wertb
	add AL, R20
wertb:
	cpi AL, 200				;Wert nicht ber Tabellenende zulassen
	brcs wt1
	ldi AL, 200				;letzter Wert = 100%
wt1:
	sts ist, AL				;neuer ist Wert
	clr AH
;Prfe, ob linear oder von Tabelle
	lds R18, Kurve	
	sbrs R18, 0				;lin oder log
	rjmp WertL				;lin fr Glhbirnen
	clc						;Clear Carry
	clr AH
	rol AL					; *2
	rol AH
	ldiw z, Taba*2
	add R30, AL
	adc R31, AH				;bertrag
	lpm AL, z+
	lpm AH, z
;	com AL					;damit 0 aus ist
;	com AH
	clz						;Clear Zero Flag
	ret
wt2:
	sez						;Set Zero Flag
	ret
;-----------------------------------------------
WertP:						;R20=%
	clr R18
	clr R19
	clr R22
	clr R23
;*Wert
	rcall umul32			;mal	 R19..R16 = R19..R16 * R23..R20 (Ganzzahlmultiplikation)
;/200
	ldi R20, 200
	clr R21
	clr R22
	clr R23
	rcall urdiv32			;durch	 R19..R16 = R19..R16 / R23..R20 (Ganzzahldivision)
	ret
;-----------------------------------------------
WertL:						;R21,R22 0..400 --> AL,AH 0..65535	- 65535*R20/400 - Linearer Verlauf
	ldi R20,LOW (fosc)
	ldi R21,HIGH (fosc)
;	ldi R20, 0xff			;R23..R20 =0x00ff
;	ldi R21, 0xff
	clr R22
	clr R23
	clr R18
	clr R19
;*Wert
	rcall umul32			;mal	 R19..R16 = R19..R16 * R23..R20 (Ganzzahlmultiplikation)
;/400
	ldi R20, 0xc8			;0xc8 =200
	ldi R21, 0x00
	clr R22
	clr R23
	rcall urdiv32			;durch	 R19..R16 = R19..R16 / R23..R20 (Ganzzahldivision)
;	com AL					;damit 0 aus ist
;	com AH
	clz						;Clear Zero Flag
	ret						;fertig
;-------------
; soll-ist- Vergleich
sollist:
; Soll ist Vergleich fr den 1. PWM	
	lds AL, soll1
	sts soll, AL
	lds R18, dk
	sts Kurve, R18
	lds AH, ist1			;soll ist Vergleich
	sts ist, AH				;soll ist Vergleich
	lds R19, dmin1
	sts dmin, R19	
	rcall wert				;wert aus Tabelle lesen
	breq S1a				;keine nderung
	in AL, TCCR0A			;
	tst AH					;Teste, ob 0
	brne s11
	andi AL, 0b01111111		;Bit7 =0
	rjmp s12
s11:
	ori AL, 0b10000000		;Bit7 =1
s12:
	out TCCR0A, AL			;zurck schreiben
	out ocr0A, AH
	lds AL, ist
	sts ist1, AL
	brtc S2
	clt						;lsche Flag
	rjmp S2
S1a:
	mov AL, R14
	andi AL, 0b00001110
	mov R14, AL
S2:
; Soll ist Vergleich fr den 2. PWM	
	lds AL, soll2
	sts soll, AL
	lds R18, dk
	lsr R18					;Bit1 nach rechts
	sts Kurve, R18
	lds AH, ist2			;soll ist Vergleich
	sts ist, AH				;soll ist Vergleich
	lds R19, dmin2
	sts dmin, R19
	rcall wert				;wert aus Tabelle lesen
	breq S2a				;keine nderung
	in AL, TCCR0A			;
	tst AH					;Teste, ob 0
	brne s21
	andi AL, 0b11011111		;Bit5 =0
	rjmp s22
s21:
	ori AL, 0b00100000		;Bit5 =1
s22:
	out TCCR0A, AL			;zurck schreiben
	out ocr0B, AH
	lds AL, ist
	sts ist2, AL
	brtc S3
	clt						;lsche Flag
	rjmp S3
S2a:
	mov AL, R14
	andi AL, 0b00001101
	mov R14, AL
S3:
; Soll ist Vergleich fr den 3. PWM	
	lds AL, soll3
	sts soll, AL
	lds R18, dk
	lsr R18					;Bit2 nach rechts
	lsr R18
	sts Kurve, R18
	lds AH, ist3			;soll ist Vergleich
	sts ist, AH				;soll ist Vergleich
	lds R19, dmin3
	sts dmin, R19					
	rcall wert				;wert aus Tabelle lesen
	breq S3a				;keine nderung
	in R18, TCCR1A			;
	tst AH					;Teste, ob 0
	brne s31
	tst AL					;Teste, ob 0
	brne s31
	andi R18, 0b01111111	;Bit7 =0
	rjmp s32
s31:
	ori R18, 0b10000000		;Bit7 =1
s32:
	out TCCR1A, R18			;zurck schreiben
;	cli						;glob INT ausschalten - keine Unterbrechung beim Schreiben der 2 Bytes
	out OCR1AH, AH			
	out OCR1AL, AL
	lds AL, ist
	sts ist3, AL
;	sei						;glob INT einschalten
	brtc S4
	clt						;lsche Flag
	rjmp S4
S3a:
	mov AL, R14
	andi AL, 0b00001011
	mov R14, AL
S4:
; Soll ist Vergleich fr den 4. PWM	
	lds AL, soll4
	sts soll, AL
	lds R18, dk
	lsr R18					;Bit3 nach rechts
	lsr R18
	lsr R18
	sts Kurve, R18
	lds AH, ist4			;soll ist Vergleich
	sts ist, AH				;soll ist Vergleich
	lds R19, dmin4
	sts dmin, R19
	rcall wert				;wert aus Tabelle lesen
	breq S4a				;keine nderung
	in R18, TCCR1A			;
	tst AH					;Teste, ob 0
	brne s41
	tst AL					;Teste, ob 0
	brne s41
	andi R18, 0b11011111	;Bit5 =0  - Port durchschalten
	rjmp s42
s41:
	ori R18, 0b00100000		;Bit5 =1  - Dimmer ein
s42:
	out TCCR1A, R18			;zurck schreiben
;	cli						;glob INT ausschalten
	out ocr1BH, AH
	out ocr1BL, AL
;	sei						;glob INT einschalten
	lds R18, ist
	sts ist4, R18
	brtc S5
	clt						;lsche Flag
	rjmp S5
S4a:
	mov AL, R14
	andi AL, 0b00000111
	mov R14, AL
S5:
; Soll ist Vergleich fr den 1.Taster
	lds AL, soll5
	cpi AL, 2				;ber 1 ?
	brcc S7					;dann berspringen
	andi AL, 0b00000001	
.if PWM_Taste
	brne S5a
.else
	breq S5a
.endif
S5e:						;Bit 0 setzen	
	sbis PINB, 0
	sbi PORTB, 0			;setzee Bit 0
	rjmp S6
S5a:
	cbi PORTB, 0			;lsche Bit 0									
S6:
S7:
; Soll ist Vergleich fr den 2.Taster		
	lds AL, soll6
	cpi AL, 2				;ber 1 ?
	brcc S8					;dann berspringen
	andi AL, 0b00000001	
.if PWM_Taste
;	breq S7a
	brne S7a
.else
;	brne S7a
	breq S7a
.endif
S7e:						;Bit 0 setzen	
	sbis PINB, 1
	sbi PORTB, 1			;setzee Bit 0
	rjmp S8
S7a:
	cbi PORTB, 1			;lsche Bit 0
S8:
.if Quarz
.else
	sbic PINA, 0			;Pin abfragen
	rcall ein				;Zentral Ein nur, wenn kein Quarzosc angeschlossen ist
.endif
	sbic PINA, 1			;Pin abfragen
	rcall aus				;Zentral aus
	ret
;-------------------------------------------------------------
Uhr:
	clr AL
	sts Zeit, AL
	lds AL, nminute
	sbrs AL, 0				;Sprung, wenn Bit gesetzt
	ret
	andi Al, 0b11111110		;Bit 0 lchen
	sts nminute, AL
	lds AL, Minute			;lese Minute
	inc AL
	cpi AL, 60				;nchste Stunde? 60=0x3C
	breq Uhr2				;nchste Stunde
	sts Minute, AL			;speichern
	ldi AL, 0b00000001		;Bit fr Minute
	sts Zeit, AL			;Marke nchste Minuten Bit 0
Uhr0:
	ret
Uhr2:						; Minute auf 0 und Stunde +1	
	ldi AL, 0				;neue Stunde = Minutenwert auf 0
	sts Minute, AL			;speichern
	lds AL, Stunde			;lese Stunde
	inc AL
	cpi AL, 24				;nchste Stunde? 24=0x18
	breq Uhr3				;nchster Tag
	sts Stunde, AL			;speichern
	ldi AL, 0b00000011		;Bit fr Stunden
	sts Zeit, AL			;Marke nchste Minuten Bit 0
	ret
Uhr3:						; Stunden auf 0 und Tage +1
	ldi AL, 0
	sts Stunde, AL			;speichern

	lds AL, Wochentag		;lese Wochentag
	inc AL
	andi AL, 7				;max 7 und 00000111b
	brne Uhr3a
	inc AL					;0+1
Uhr3a:
	sts Wochentag, AL 		;wochentag +1
	ldiw z, MTage*2-1		;Ermittle Tage des Monats aus Liste
	lds AH, Monat			;lese Monat
	add R30, AH				;low Z
	lpm AH,z				;Wieviel Tage hat der Monat
	lds AL, Tag				;aktueller Tag

	cp AL, AH				;nchster Tag? 31=0x1f
	brcc Uhr4				;nchster Monat
	inc AL
	sts Tag, AL				;speichern
	ldi AL, 0b00000111		;Bit fr Tage
	sts Zeit, AL			;Marke nchste Minuten Bit 0
	ret
Uhr4:						;Tag auf 0 und Monat +1
	ldi AL, 1
	sts Tag, AL				;speichern
	lds AL, Monat			;aktueller Monat
	inc AL
	cpi AL, 13				;nchster Monat?
	breq Uhr5				;letzter Monat
	sts Monat, AL			;speichern
	ldi AL, 0b00001111		;Bit fr Monate
	ret
Uhr5:						;Neues Jahr Monat 1
	ldi AL, 1
	sts Monat, AL			;speichern
	ldi AL, 0b00001111		;Bit fr Monate
	sts Zeit, AL			;Marke nchste Minuten Bit 0
	ret
MTage:
	.db 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31	;Tage/Monat
;-------------------------------------------------------------
;Status
;0= nach Reset oder Neustart oder kein Favorit
;1= Favorit gewhlt INT an Port PB6
;2= Zentral aus ausgefhrt
;3= Zentral ein ausgefhrt
;4= Anweisung Favorit lesen - Favorit 0 = alles aus
;5= Anweisung Favorit schreiben
;6= Maximalwert in EEPROM schreiben
;7= Minimalwert fr 8-Bit Dimmer berechnen
;8= Anweisung ausgefhrt
;9= Werte durch Max begrenzt
;10 = Flag Bedingt
;12 bis 19 = Dimmer 1 bis 4 setzen
;20 Schaltzeiten lesen
;21 Schaltzeiten speichern
;22 Schaltzeiten sortieren
;23 Favorit anzeigen

setstatus:
	lds AL, status
	cpi AL, 0
	brne setstatus1			;Status 0  - zurck	
	rjmp setstatusx
setstatus1:
	cpi AL, 4				;Anweisung Favorit aus EEPROM lesen
	brne setstatus4			;andere Anweisung
setstatus1a:

	rcall setstatusc

	rcall alt_nach_neu
	rjmp sx
setstatus4x:
	rcall aus				;Favorit 0 = alles aus
	ldi AL, 0
 	sts status, AL
	ret
setstatus4:
	cpi AL, 5				;Anweisung Favorit in EEPROM schreiben
	brne setstatus5			;andere Anweisung
setstatus4a:
	lds AL, favorit
	cpi AL, 0				;vergleichen 0
	breq setstatus5x
	cpi AL, 17				;vergleichen bis 16
	brcc setstatus5x
	rcall est				;fav-Nr in AL - Adresse in AH zurck
	ldiw X, soll1			;ziel
	rcall schreiben			;AH = Adresse im EEPROM - Doppelregister X = Ziel
	rjmp sx
setstatus5x:				;(4x)
	ldi AL, 0
 	sts status, AL
	ret
setstatus5:
	cpi AL, 6				;Minimalwert ins EEPROM
	brne setstatus6			;andere Anweisung
setmax:
	ldiw X, dmin1			;Doppelregister X = Quelle ---> AH = Ziel im EEPROM
	ldi AH, minimal
	rcall schreiben			;Werte ins EEPROM schreiben
	rjmp sx
setstatus6:
	cpi AL, 7				;Favorit anzeigen
	breq setstatus6a
	rjmp setstatus7
setstatus6a:
	rcall setstatusc
	rjmp sx
setstatusc:
	lds AL, favorit
	cpi AL, 0				;vergleichen 0
	breq setstatus4x
	cpi AL, 17				;vergleichen >16
	brcc setstatus5x
	rcall est				;fav-Nr in AL - Adresse in AH zurck
	ldiw X, soll1			;ziel
	rcall lesen				;AH = Adresse im EEPROM - Doppelregister X = Ziel
	ret

setstatusf:
	ldi AL, 9				;Fehler
 	sts status, AL
	ret 
setstatus7:
	cpi AL, 20				;AL>19?
	brcc setstatus11
	cpi AL, 12
	brcs setstatus11

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------
		lds R18, dk					;lade etabd - dimmkurven 12 bis 19
		cpi AL, 12
		brne setdk1
		andi R18, 0b11111110		;Bit0 auf 0
setdk1:
		cpi AL, 13
		brne setdk2
		ori R18, 0b00000001			;Bit0 auf 1
setdk2:
		cpi AL, 14
		brne setdk3
		andi R18, 0b11111101		;Bit1 auf 0
setdk3:
		cpi AL, 15
		brne setdk4
		ori R18, 0b00000010			;Bit1 auf 1
setdk4:
		cpi AL, 16
		brne setdk5
		andi R18, 0b11111011		;Bit2 auf 0
setdk5:
		cpi AL, 17
		brne setdk6
		ori R18, 0b00000100		;Bit2 auf 1
setdk6:
		cpi AL, 18
		brne setdk7
		andi R18, 0b11110111		;Bit3 auf 0
setdk7:
		cpi AL, 19
		brne setdk8
		ori R18, 0b00001000			;Bit3 auf 1
setdk8:
		andi R18, 0x0f				;0000xxxx
		mov AL, R18
		swap AL
		com AL
		andi AL, 0xf0				;0000xxxx
		or AL, R18



	ldi AH, etabd			;Adresse Dimmkurve
	rcall weprom			;R17 Adresse R16 Wert - ins EPROM schreiben
	rcall DKS				;Dimmkurve aus EEPROM in RAM - 0 bis 4
		ldi R18, 8
		sts status, R18			;Status 8

	rjmp sx



setstatus11:				;Nulldurchgang 50 Hz
	cpi AL, 11				;Rert fr Timer
	brne setstatus12
	lds AL, favorit
	sts CWert, AL
	ldi AH, ECWert			;R17 Adresse R16 Wert - ins EPROM schreiben
	rcall weprom
	rjmp sx
setstatus12:
;Zeiten fr Schaltuhr
setstatus13:
	cpi AL, 20				;Werte aus EEPROM neu laden
	brne setstatus14
	rcall lesen2
	rjmp sx
setstatus14:
	cpi AL, 21				;Werte in EEPROM neu schreiben
	brne setstatus15		;letzte Anweisung  - Fehler
	rcall schreiben2
	rjmp sx
setstatus15:
	cpi AL, 22				;Werte in EEPROM neu schreiben
	brne setstatusx			;letzte Anweisung  - Fehler
	rcall sort

;hier weitere Abfragen einsetzen
sx:
	ldi AL, 8				;erledigt
 	sts status, AL
setstatusx:
	ret
est:						;Fav in AL bergeben
	clr R18					;zhler 0...fav
	ldi R19, 6				;+Adresse fav
	clr AH					;erste Adresse
est1:
	inc R18
	cp R18, AL				;6 Werte in EEPROM
	breq estx
	add AH, R19
	rjmp est1
estx:
	ret						;in AH steht die Adresse
;-------------
;ende Status setzen
aus:
	ldi AL, 0x00			;letzter Wert = 0
	ldi AH, 0
	rcall ae

	ldi AL, 2
	sts status, AL			;Status 0
	ret
ein:
	ldi AL, 100				; maximalwert
	ldi AH, 1
	rcall ae
	ldi AL, 3
	sts status, AL			;Status 0
	ret
ae:
	sts soll1, AL
	sts soll2, AL
	sts soll3, AL
	sts soll4, AL
	sts soll5, AH			;Ausgnge Taster aus
	sts soll6, AH
	ret
;---------------------------------
auto:						;Schaltzeituhr
	lds AL, Zeit
	sbrs AL, 0				;Sprung, wenn Bit gesetzt
	ret
	Ldi R20, 32				;Schleifenzhler
	ldiw Z, WF0				;Anfang Tabelle Schaltzeiten
auto1:
	subi R20, 1
	brcc auto1z
	ret
auto1z:
	rcall AutoZ				;prfe Datensatz
	rcall Auto0
	adiw z, 6
	rjmp auto1
Auto0:											
	lds AL, Minute
	ldd AH, Z+1
	cp AL, AH				;Vergleiche Minute
	breq Auto0a
	ret
Auto0a:
	lds AL, Stunde
	ldd AH, Z+2
	cp AL, AH				;Vergleiche Stunde
	breq Autob0
	ret						;liefert Z bei Gleichheit
Autob0:
;Wochentag als Bitfolge erzeugen
	clr AH					;Maske Wochenzeit Bit0 = Montag - bis Bit7= Sonntag  ;ist Wochentag =0 - also nicht gesetzt, werden alle Bits auf 0 gesetzt
	lds AL, Wochentag		;Wert 1 bis 7
	andi AL, 0x07			;und 00000111  - die rechten 3 Bits werden gefiltert
	breq autod				;Wochentag =0 jetzt zum Datum
	sec						;set Carry
Autob0b:
	rol AH
	dec AL
	clc
	breq Autob0a
	rjmp Autob0b
Autob0a:
;Wochentag vergleichen
	ld AL, Z
	and AH, AL
	breq autod				;kein Wochentag - weiter zum Datum
	ld AL, Z
	sbrs AL, 7				;Prfe Bit7
	rjmp autod3				;schalte bedingungslos, wenn Monat stimmt
	lds AH, status			;Prfe, ob Status = 10
	cpi AH, 10
	breq autod3				;schalte bedingt, wenn Monat stimmt
	ret						;Wochentag stimmt aber kein Fav 10
auto0b:						;Fav senden
	ldd AL, Z+5				;Fav laden
	sts Favorit, AL
	rcall setstatus1a		;werte aus EPROM lesen 
	ldi AL, 10				;Status = 10
	sts status, AL
	ldi R20, 0				;Ende der Suche - keine weiteren Durchlufe
	ret						;fertig
autod:						;Tag und Monat abfragen
	ldd AL, Z+3				;Tag laden
	cpi AL, 0				;Tag =0?
	brne autod1
	ret						;Tag =0 zurck
autod1:
	cpi AL, 32				;Tag >31?
	brne autod2
	ldi AL, 0				;Tag >31 --->Tag 0
	std Z+3, AL				;Tag = 0
	std Z+4, AL				;Monat = 0
	ret						;fertig
autod2:						;richtige Tag?
	lds AH, Tag
	cp AL, AH
	breq auto0b				;richtiger Tag - Fav schreiben
	ret						;falsche Tag
autod3:						;richtiger Tag -Monat abfragen
	ldd AL, Z+4				;Monat laden
	cpi AL, 0				;Monat =0?
	breq auto0b				;jeder Monat also senden
	cpi AL, 13				;Monat >12?
	brcs autod4
	ldi AL, 0
	std Z+4, AL				;Monat = 0
	ret						;fertig
autod4:						;richtiger Monat?
	lds AH, Monat
	cp AL, AH
	breq auto0b				;richtiger Monat ---> senden
	ret
AutoZ:						;ungltige Zeit - Daten auf 0
	ldd AL, Z+1
	cpi AL, 60				;Stunde
	brcc AZ
	ldd AL, Z+2
	cpi AL, 24				;Minute
	brcc AZ
	ldd AL, Z+3
	cpi AL, 32				;Datum
	brcc AZ
	ldd AL, Z+4
	cpi AL, 13				;Monat
	brcc AZ
	ret
AZ:
	clr AL
	st Z, AL
	std Z+1, AL
	std Z+2, AL
	std Z+3, AL
	std Z+4, AL
	std Z+5, AL
	ret
;---------------------------------
schreiben:
;ab soll1 --> daten_eeprom
	ldi R18, 0				;Schleifenzhler auf 0 
schreiben1:	 				;Schleifenzhler R18 auf 2 = 4 Werte schreiben
	cli						;glob INT ausschalten
schreiben1a:	
	ld AL, X+
	rcall weprom
	inc AH
	inc R18
	cpi R18, 6				;6 Werte in EEPROM 
	brlo schreiben1a
	sei						;glob INT einschalten
	ret
schreiben2:					;schreibt die Schaltzeiten bei ungleichheit ins EEPROM
	cli						;glob INT ausschalten
	ldi AH, EWF0			;Adresse Schaltzeit im EEPROM
	ldiw X, WF0				;Adresse Schaltzeit im SRAM
	ldi R18, 0
schreiben2b:
	rcall reprom			;lesen
	ld R19, X+
	CP R19, AL
	breq schreiben2a		;Werte sind gleich nicht neu schreiben
	mov AL, R19				;Wert schreiben
	rcall weprom
schreiben2a:
	inc AH					;AH = Adresse im EEPROM
	inc R18					;Schleifenzhler
	cpi R18, 144			;144 Werte - 24*6Byte
	brlo schreiben2b
	sei						;glob INT einschalten
	ret
;---------------------------------
;Werte von EEPROM in SRAM
;
lesen:
;AH = Adresse im EEPROM - Doppelregister X = Ziel
	ldi R18, 0				;Schleifenzhler auf 0 
lesen1:						;Schleifenzhler R18 auf 2 = 4 Werte lesen 
	rcall reprom
	st X+, AL
	inc AH					;AH = Adresse im EEPROM
	inc R18					;Schleifenzhler
	cpi R18, 6				;6 Werte 
	brlo lesen1
	ret
lesen2:						;liest Schaltzeiten vom EEPROM und schreidt sie ins RAM
	ldi AH, EWF0			;Adresse Schaltzeit im EEPROM
	ldiw X, WF0				;Adresse Schaltzeit im SRAM
	ldi R18, 0
lesen2a:
	rcall reprom			;lesen
	st x+, AL
	inc R18
	inc AH
	cpi R18, 144			;144 Werte - 24*6Byte
	brlo lesen2a
	ret
;---------------------------------
; *** EEPROM lesen und schreiben ***
; reprom = Read EEPROM areg(R17)=Adresse Rckgabe R16 <= Inhalt
reprom:	
	sbic	EECR, EEPE
	rjmp	reprom
	out	EEAR, r17			; Adressregister <= Adresse
	sbi	EECR, EERE			; Lesebit setzen
	in	r16, EEDR			; R16 <= Inhalt
	ret
; weprom = Write EEPROM areg(R17)=Adresse R16=Inhalt keine Rckgabe
weprom:	
weprom1:
	sbic EECR, EEPE
	rjmp	weprom1
	out	EEAR, r17			; Adressregister <= Adresse
	out	EEDR, r16			; Datenregister <= Inhalt
	sbi EECR, 2				;EEMPE	Bit 2  EEMPE: EEPROM Master Program Enable - Schreiben vorbereiten
	sbi	EECR, 1				;EEWE	EEWE=1 schreiben - Bit 1  EEPE: EEPROM Program Enable - Schreiben
	ret
;-----

;Setze Fav16 auf 100, wenn Werte >100
SF16:
	ldi AH, favorit16		;1. Dimmer
	rcall SF16a
	ldi AH, favorit16+1		;2. Dimmer
	rcall SF16a
	ldi AH, favorit16+2		;3. Dimmer
	rcall SF16a
	ldi AH, favorit16+3		;4. Dimmer
	rcall SF16a
	ldi AH, favorit16+4
	ldi AL, 1				;1. Schalter an
	rcall weprom
	ldi AH, favorit16+5
	ldi AL, 1				;2. Schalter an
	rcall weprom
	ret
SF16a:
	rcall reprom			;lese Wert
	cpi AL, 100				;prfe, ob >100
	brcs SF16b
	ldi AL, 100
SF16b:
	rcall weprom
	ret


DKS:
;Dimmkurve aus EEPROM in RAM - 0 bis 4
	cli					;glob INT ausschalten								noch keine Int eingerichtet
	ldi AH, etabd
	rcall reprom
	cpi AL, 0xFF		;leeres EEPROM?
	brne DKS1
	ldi AL, 0x0F		;alle LEDs
DKS1:
	sts dk, AL
	sei					;glob INT einschalten
	ret
;----------------------------------------------------------------------------------------------------------
Init:
	ldi AH, ECWert
	rcall reprom
	cpi AL, 0xFF
	brne Init1
	ldi AL, OffSync
Init1:
	sts CWert, AL
	rcall lesen2			;Daten von EEPROM --> SRAM
	clr AL					;Rest mit 0 fllen
Init2:
	st X+,AL
	inc R18					;Schleifenzhler
	cpi R18, 192
	brlo Init2
	ldi AL, 0x01			;Richtung Dimmen voreinstellung hochdimmen
	mov R9 , AL				;Dimmer 1
	mov R10, AL				;Dimmer 2
	mov R11, AL				;Dimmer 3
	mov R12, AL				;Dimmer 4
	rcall SF16				;setze Fav16 auf max 100 und beide Schalter an

							;minimalwerte aus EEPROM  lesen
	ldi R18, 0				;Minimale Helligkeit aus dem EPROM lesen
	ldi AH, minimal			;AH = Adresse im EEPROM - Doppelregister X = Ziel
	ldiw X, dmin1
	rcall reprom
	cpi AL, 101				;Anfangshelligkeit beim 8-Bit Timer
	brcs Init3
	ldi AL, 0
Init3:
	st X+, AL
	inc AH					;Adresse im EEPROM +1
	rcall reprom
	cpi AL, 101				;Anfangshelligkeit beim 8-Bit Timer
	brcs Init4
	ldi AL, 0
Init4:
	st X+, AL
	inc AH					;Adresse im EEPROM +1
	rcall reprom
	cpi AL, 101				;Anfangshelligkeit beim 16-Bit Timer
	brcs Init5
	ldi AL, 0
Init5:
	st X+, AL
	inc AH					;Adresse im EEPROM +1
	rcall reprom
	cpi AL, 101				;Anfangshelligkeit beim 16-Bit Timer
	brcs Init6
	ldi AL, 0
Init6:
	st X+, AL
	clr AL
	st X+, AL
	st X+, AL
	ldi AL, 0b10000000		;Marke fr Uhrzeit neu setzen
	sts nminute, AL

	ldi AL, 0				;Zeit auf 0 stellen
	sts Minute, AL
	sts Stunde, AL
	sts Tag, AL
	sts Monat, AL
	sts Wochentag, AL
;-----
	sbi ACSR,ACD			;disable AnalogCoparator
	outi PCMSK, 0b01000000			;Pin Change Mask Register 0 PB6 maskieren fr 50Hz sync genutzt
;	outi GIMSK, 0b00010000

;	outi GIMSK, 0b00000000			; Interrupt INT0, INT1 sperren
	outi GIMSK, 0b00111111		; Interrupt INT0, INT1 sperren PCIE0  fr Port B ein
;	outi MCUCR,(1<<SE)				;Sleep On///Power-Down Mode
	in AL, MCUCR
	cbr AL, (1<< SM0)|(1<< SM1)		;0 0
	sbr AL, (1<< SE)				;1
	out MCUCR, AL
	ldi AL, 0b01000000
	out PCMSK, AL
	in AL, GIMSK
	sbr AL, (1<< PCIE)
	out GIMSK, AL
;I2Cinit:
	outi USICR,(1<<USISIE)|(1<<USIWM1)|(0<<USIWM0)|(1<<USICS1)	;| ;Start Condition Interrupt;Set USI in Two-wire mode;Clock Source = External, positive edge        */  \
    outi USISR,(1<<USISIF)|(1<<USIOIF)|(1<<USIPF)|(1<<USIDC)	;Clear all flags
	outi Flag,0
	outi DDRB, 0b10011111				;Daten Richtungsregister Ausgang = 1
	outi DDRD, 0b00100000
	outi PORTB,(1<<SCL)+(1<<6)+(1<<SDA)			;Port 1=pull-up aktiviert
	outi DDRA, 0b00000000																				; TEST !!!!!!!!!!!!!!!!!!
	ldi	r16, 0x00		; Port D
	out	PORTD, r16		; pull-up deaktiviert
;PWM Init

;Timer
	ldi AL, k_vorteiler1	;Vorteiler  8Bit einstellen
	out tccr0B, AL			;fr  8 Bit Counter
	ldi AL, k_vorteiler2	;Vorteiler 16Bit einstellen
	out tccr1B, AL			;fr 16 Bit Counter	

	ldi AL, k_pwmmode1		;Fast PWM-Mode 3 und 
	out tccr0a, AL			;clear OCA0 on Compare  ;tccr0a tccr0b --> Set OC0A on Compare Match, clear OC0A at TOP

	ldi AL, k_pwmmode2		;COM1A1 COM1A0 COM1B1 COM1B0   WGM11 WGM10
	out tccr1a, AL			;clear OCA0 on Compare  ;tccr0a tccr0b --> Set OC0A on Compare Match, clear OC0A at TOP

	ldi AL, k_TIMSK			;Interupt Timer A
	out TIMSK, AL			;fr  8 Bit Counter	
	ldi AL,HIGH (fosc)		;Oscillator Frequenz
	OUT ICR1H,AL
	ldi AL,LOW (fosc)
	OUT ICR1L,AL

.if PWM8
	clr AL		
.else
	ldi AL, 255			
.endif
	out ocr0A, AL
	out ocr0B, AL
.if PWM16
	clr AL		
.else
	ldi AL, 255			
.endif
	out OCR1AH, AL			
	out OCR1AL, AL
	out OCR1BH, AL			
	out OCR1BL, AL
;-----
	rcall DKS				;Dimmkurve aus EEPROM in RAM - 0 bis 4

i5:
.if WatchdogT
	wdr
	nop
	ldi AL, (0<<WDRF)
	out MCUSR, AL
	in AL, WDTCSR
	ori r16, (1<<WDIE)|(1<<WDCE)|(1<<WDE)|(1<<WDP3)|(0<<WDP2)|(0<<WDP1)|(1<<WDP0)
	out WDTCSR, AL
	ldi AL, (0<<WDE)	;WDR aus
	out WDTCSR, AL
;Init Watchdog-Timer
	;	outi WDTCR, 0b01001111		; -->2 sec
;	WDTCSR					WDIF WDIE WDP3 WDCE WDE WDP2 WDP1 WDP0
							;WDP3 WDP2 WDP1 WDP0
							;  0    1    1    1    256K  cycles 2.0 s
							;  1    0    0    1    1024K cycles 8.0 s
	.endif
	sei
	ret
; ende Init:
WDR_RESET:
	rcall init
	ldi AL, 99
	sts favorit, AL			;Status 99
RESET:
	rcall init
	rcall aus

MainLoop:
.if WatchdogT
	WDR
.endif
	rcall Uhr
	rcall sollist
	rcall auto						;Zeitschaltuhr schalten
	rcall warte
	rcall td						;Tastenabfrage Dimmer
	rcall setstatus
	rjmp MainLoop

sort:						;sortiere nach Uhrzeit
	ldiw Y, WF0				;1. Adresse Schaltzeit im SRAM
	ldiw Z, WF0+6			;2. Adresse Schaltzeit im SRAM
	clr R21					;Datensatz Zhler
	clr R22					;Marker fr Sortieren
sort1:
	cpi R21, 23				;24 Datenstze
	brcs sorta
	tst R22					;Marker gesetzt?
	brne sort				;neuer Durchlauf bis kein Tausch gemacht wird
	ldi AL, 8				;Status = 8
	sts status, AL
	ret						;fertig - zurck

sorta:						;Aktiv+Wochentag/Minuten/Stunden/Monat/Tag/Faforit
	ldd AL, Y+1				;Uhrzeit
	ldd AH, Y+2
	tst AH					;=0?
	brne sort2
	tst AL
	brne sort2
	ldi AL, 60
	ldi AH, 60
sort2:
	ldd R18, Z+1
	ldd R19, Z+2
	tst R19					;=0?
	brne sort3
	tst R18
	brne sort3
	ldi R18, 60
	ldi R19, 60
sort3:				
	cp AH, R19				;Stunden gleich?
	breq sort5
	brcc tausch
	rjmp sort6
	sort7:
	tst R18
	brne tausch
	rjmp sort6
sort5:
	cp AL, R18				;Minuten gleich?
	breq sort6
	brcc tausch
sort6:
	adiw Y, 6
	adiw Z, 6
	inc R21
	rjmp sort1

tausch:
	clr R20
tausch2:
	ld AL, Y
	ld AH, Z
	st Y+, AH
	st Z+, AL
	inc R20
	cpi R20, 6
	breq tausch1			;nchster Datensatz
	rjmp tausch2
tausch1:
	inc R21					;Datensatz Zhler
	ldi R22, 1				;Marker fr Tausch setzen
	rjmp sort1

;-----------------------------------------------
;rcall umul32			;mal	 R19..R16 = R19..R16 / R23..R20 (Ganzzahlmultiplikation)
;rcall urdiv32			;durch	 R19..R16 = R19..R16 / R23..R20 (Ganzzahldivision)

;https://www.mikrocontroller.net/articles/AVR_Arithmetik#32_Bit_.2F_32_Bit
umul32:					;32 Bit * 16 Bit
	cpi	R19, 0
	cpc	R19, R18
	breq	_umu1		;one operand must be below 65536
	mov	R24, R16		; swap A <-> B
	mov	R16, R20
	mov	R20, R24
	mov	R24, R17
	mov	R17, R21
	mov	R21, R24
	mov	R22, R18
	mov	R23, R19
	clr	R18
	clr	R19
;				a3,2,1,0 = a1,0 * b3,2,1,0
;				R19,R18,R17,R16 = R17,R16 * R23,R22,R21,R20
_umu1:	ldi	R28, 16
	clr	R24
	clr	R25
	ror	R17
	ror	R16
_umu2:	brcc	_umu3
	add	R18, R20
	adc	R19, R21
	adc	R24, R22
	adc	R25, R23
_umu3:	ror	R25
	ror	R24
	ror	R19
	ror	R18
	ror	R17
	ror	R16
	dec	R28
	brne	_umu2
	ret

;-----------------------------------------------
urdiv32:			;32 Bit / 32 Bit
; a3..0 = a3..0 / b3..0 (Ganzzahldivision)
; R19..R16 = R19..R16 / R23..R20 (Ganzzahldivision)
; b3..0 = a3..0 % b3..0 (Rest)
; R23..R20 = R19..R16 % R23..R20 (Rest)
; cycle: max 684
	mov	R24, R20		;T = B
	mov	R25, R21
	mov	R26, R22
	mov	R27, R23
	lsr	R27		;B / 2
	ror	R26
	ror	R25
	ror	R24
	add	R16, R24		;A = A + B / 2
	adc	R17, R25
	adc	R18, R26
	adc	R19, R27
udiv32:
	clr	R24
	clr	R25
	clr	R26
	clr	R27
	ldi	R28, 32
udi1:	lsl	R16
	rol	R17
	rol	R18
	rol	R19
	rol	R24
	rol	R25
	rol	R26
	rol	R27
	cp	R24, R20
	cpc	R25, R21
	cpc	R26, R22
	cpc	R27, R23
	brcs	udi2
	sub	R24, R20
	sbc	R25, R21
	sbc	R26, R22
	sbc	R27, R23
	inc	R16
udi2:	dec	R28
	brne	udi1
	mov	R20, R24
	mov	R21, R25
	mov	R22, R26
	mov	R23, R27
	ret
;----------------------------------------------------------------------------------------------------------
;.db "Anfang der Tabelle"
Taba:	;Verlauf fr LED
 .if Osc
.dw 0																								;erster Wert 0 = aus
.dw 256,263,270,278,286,294,302,311,319,328,338,347,357,367,378,388,399,411,422,434
.dw 446,459,472,485,499,513,528,543,558,574,590,607,624,642,660,678,698,717,738,758
.dw 780,802,825,848,872,897,922,948,975,1002,1031,1060,1090,1121,1152,1185,1218,1253,1288,1325
.dw 1362,1401,1440,1481,1523,1566,1610,1655,1702,1750,1800,1851,1903,1957,2012,2069,2127,2188,2249,2313
.dw 2378,2446,2515,2586,2659,2734,2811,2891,2972,3056,3143,3232,3323,3417,3513,3613,3715,3820,3928,4039
.dw 4153,4270,4391,4515,4643,4774,4909,5047,5190,5337,5488,5643,5802,5966,6135,6308,6486,6670,6858,7052
.dw 7251,7456,7667,7883,8106,8335,8571,8813,9062,9318,9582,9852,10131,10417,10711,11014,11325,11645,11974,12313
.dw 12661,13018,13386,13765,14154,14554,14965,15388,15822,16270,16729,17202,17688,18188,18702,19230,19774,20333,20907,21498
.dw 22105,22730,23372,24033,24712,25410,26128,26867,27626,28406,29209,30034,30883,31756,32653,33576,34525,35500,36503,37535
.dw 38595,39686,40807,41961,43146,44365,45619,46908,48234,49597,50998,52439,53921,55445,57011,58622,60279,61982,63734,65535

.else
.dw 0																								;erster Wert 0 = aus
.dw 256,263,270,277,285,293,301,310,318,327,336,346,355,365,375,386,396,408,419,431
.dw 443,455,467,481,494,508,522,536,551,567,582,598,615,632,650,668,687,706,725,745
.dw 766,787,809,832,855,879,903,928,954,981,1008,1036,1065,1095,1125,1156,1188,1221,1255,1290
.dw 1326,1363,1401,1440,1480,1521,1564,1607,1652,1698,1745,1793,1843,1894,1947,2001,2057,2114,2173,2233
.dw 2295,2359,2425,2492,2562,2633,2706,2781,2859,2938,3020,3104,3190,3279,3370,3464,3560,3659,3761,3865
.dw 3973,4083,4197,4313,4433,4557,4683,4814,4947,5085,5226,5372,5521,5675,5832,5994,6161,6332,6508,6689
.dw 6875,7067,7263,7465,7673,7886,8105,8330,8562,8800,9045,9296,9555,9820,10093,10374,10662,10959,11264,11577
.dw 11899,12229,12569,12919,13278,13647,14027,14417,14817,15229,15653,16088,16535,16995,17467,17953,18452,18965,19492,20034
.dw 20591,21164,21752,22357,22979,23617,24274,24949,25643,26355,27088,27841,28615,29411,30228,31069,31933,32820,33733,34671
.dw 35635,36625,37644,38690,39766,40871,42008,43175,44376,45610,46878,48181,49520,50897,52312,53766,55261,56798,58377,59999
.endif
;.db "Ende der Tabelle"
Tabende:
.EXIT